import React, { useEffect, useState } from "react";
import { useParams } from "react-router-dom";
import Sidebar from "../components/Sidebar";
import api from "../api";
import VideoPlayer from './VideoPlayer';

const CoursePage = () => {
    const { courseId: courseIdStr } = useParams();
    const course_id = Number(courseIdStr);
    const [courseData, setCourseData] = useState(null);
    const [user, setUser] = useState(null);
    const [loading, setLoading] = useState(true);
    const [selectedVideo, setSelectedVideo] = useState(null);
    const [expandedModule, setExpandedModule] = useState(null);
    const [watchedVideoIds, setWatchedVideoIds] = useState([]);
    const [showCompletionPopup, setShowCompletionPopup] = useState(false);
    const auth_token = localStorage.getItem("auth_token");
    const auth_token_type = localStorage.getItem("auth_token_type");
    const token = `${auth_token_type} ${auth_token}`;

    const markWatchedVideos = (videos, progressData) => {
        const watchedVideos = new Set(); 
        let moduloLastWatched = null;
        let orderLastWatched = null;    

        for (const moduleKey of Object.keys(videos)) {
            const moduleVideos = videos[moduleKey];
            
            for (const video of moduleVideos) {
                if (video.id == progressData) {
                    moduloLastWatched = moduleKey;
                    orderLastWatched = video.order;                    
                }
            }
        }

        for (const moduleKey of Object.keys(videos)) {
            const moduleVideos = videos[moduleKey];
            if ((moduleKey) < moduloLastWatched) {
                for (const video of moduleVideos) {
                    watchedVideos.add(video.id);
                }
            } else if ((moduleKey) === moduloLastWatched) {
                for (const video of moduleVideos) {
                    if (video.order < orderLastWatched) {
                        watchedVideos.add(video.id);
                    }
                }
                break; 
            }
        }
        return Array.from(watchedVideos);
    };

    const fetchCourseProgress = async (videos) => {
        try {
            const response = await api.get(`/course-progress/${course_id}`, {
                headers: { Authorization: token },
            });

            if (response.data.result) {
                const progressData = response.data.result;
                const watchedIds = markWatchedVideos(videos, progressData.last_video_watched);
                setWatchedVideoIds(watchedIds);
            } else {
                console.log("No progress data found.");
            }
        } catch (error) {
            console.log("Error fetching course progress:", error);
        }
    };

    const updateCourseProgress = async (courseId, videoId) => {
        try {
            await api.put(`/course-progress/${courseId}`, {
                video_id: videoId,
                course_id: courseId,
            }, {
                headers: { Authorization: token },
            });
        } catch (error) {
            console.error("Error updating course progress:", error);
        }
    };

    useEffect(() => {
        const fetchUserData = async () => {
            try {
                const response = await api.get("/users/", {
                    headers: { Authorization: token },
                });
                setUser(response.data.result);
            } catch (error) {
                console.log(error);
            }
        };

        const fetchCourseData = async () => {
            try {
                const response = await api.get(`/courses/${course_id}`, {
                    headers: { Authorization: token },
                });
                setCourseData(response.data.result);

                if (response.data.result.videos && Object.keys(response.data.result.videos).length > 0) {
                    const firstModuleKey = Object.keys(response.data.result.videos)[0];
                    const firstVideo = response.data.result.videos[firstModuleKey][0];

                    setSelectedVideo(firstVideo);
                    setExpandedModule(firstModuleKey);
                }

                await fetchCourseProgress(response.data.result.videos);
            } catch (error) {
                console.log(error);
            } finally {
                setLoading(false);
            }
        };

        fetchUserData();
        fetchCourseData();
    }, [course_id, token]);

    const calculateCompletionPercentage = () => {
        if (!courseData) return 0;

        let totalVideos = 0;
        let watchedVideos = 0;

        for (const moduleKey of Object.keys(courseData.videos)) {
            const moduleVideos = courseData.videos[moduleKey];
            totalVideos += moduleVideos.length;

            watchedVideos += moduleVideos.filter(video => watchedVideoIds.includes(video.id)).length;
        }

        const completionPercentage = totalVideos > 0 ? (watchedVideos / totalVideos) * 100 : 0;

        return completionPercentage.toFixed(2);
    };

    const completeVideo = async (videoId) => {
        const nextVideo = findNextVideo(videoId);
        if (nextVideo) {
            await updateCourseProgress(course_id, nextVideo.id);
            await fetchCourseProgress(courseData.videos);
            setSelectedVideo(nextVideo);
            const updatedWatchedIds = markWatchedVideos(courseData.videos, nextVideo.id);
            setWatchedVideoIds(updatedWatchedIds);
        } else {
            setShowCompletionPopup(true);
        }
    };
    
    const findNextVideo = (currentVideoId) => {
        const modules = courseData.videos;
        let nextVideo = null;
        let foundCurrent = false;

        for (const moduleKey of Object.keys(modules)) {
            const moduleVideos = modules[moduleKey];

            for (const video of moduleVideos) {
                if (foundCurrent) {
                    nextVideo = video;
                    break;
                }
                if (video.id === currentVideoId) {
                    foundCurrent = true;
                }
            }

            if (nextVideo) {
                break;
            }
        }

        return nextVideo;
    };

    const toggleModule = (moduleKey) => {
        setExpandedModule(expandedModule === moduleKey ? null : moduleKey);
    };

    const handleVideoSelect = (video) => {
        setSelectedVideo(video);
    };

    const handleClosePopup = () => {
        setShowCompletionPopup(false);
    };

    return (
        <div className="bg-white font-euphemia h-screen w-full flex" style={{ fontFamily: 'Euphemia UCAS Bold' }}>
            {!loading && user && <Sidebar user={user} />}

            <div className="flex-1 h-full p-4 flex flex-col">
                <div className="text-3xl font-bold mb-1 text-left text-[#E8C36A]">
                    {courseData ? courseData.course_name : 'Loading...'}
                </div>
                <div className="h-1 bg-[#E8C36A] mb-4" style={{ width: '60%' }}></div>
                
                <div className="mb-4">
                    <p className="text-lg">
                        Porcentagem de conclusão: {calculateCompletionPercentage()}%
                    </p>
                </div>

                <div className="flex flex-1">
                <div className="w-1/4 p-4 flex-none">
                    {courseData && Object.keys(courseData.videos).map((moduleKey) => {
                        const videos = courseData.videos[moduleKey];
                        const isExpanded = expandedModule === moduleKey;

                        return (
                            <div key={moduleKey} className="mb-0 rounded-lg overflow-hidden shadow-lg">
                                <button
                                    onClick={() => toggleModule(moduleKey)}
                                    className={`flex justify-between items-center text-white bg-[#E8C36A] hover:bg-[#F4A163] focus:ring-4 focus:outline-none focus:ring-[#F4A163] font-medium text-lg px-5 py-2.5 w-full rounded-t-lg`}>
                                    <span>Módulo {parseInt(moduleKey)}</span>
                                    <span className={`transform transition-transform duration-200 ${isExpanded ? 'rotate-180' : ''}`}>
                                        ▼
                                    </span>
                                </button>

                                {isExpanded && (
                                    <div className={`bg-gray-100 shadow-md p-2 rounded-b-lg`}>
                                        <ul className="py-2 text-sm text-gray-700">
                                            {videos.map((video) => (
                                                <li
                                                    key={video.id}
                                                    onClick={() => handleVideoSelect(video)}
                                                    className={`flex justify-between items-center py-2 cursor-pointer hover:bg-gray-200 ${watchedVideoIds.includes(video.id) ? 'bg-gray-400' : ''}`}>
                                                    <span className="text-lg">{video.video_name}</span>
                                                    {watchedVideoIds.includes(video.id) && (
                                                        <span className="text-green-600 ml-2">✔️</span>
                                                    )}
                                                </li>
                                            ))}
                                        </ul>
                                    </div>
                                )}
                            </div>
                        );
                    })}
                </div>

                <div className="w-3/4 p-4 flex flex-col">
                    <div className="flex-1 flex flex-col">
                        {selectedVideo && (
                            <VideoPlayer video={selectedVideo} onComplete={() => completeVideo(selectedVideo.id)} />
                        )}
                    </div>
                </div>
            </div>

            </div>

            {showCompletionPopup && (
                <div className="fixed inset-0 bg-gray-500 bg-opacity-75 flex justify-center items-center">
                    <div className="bg-white rounded-lg shadow-lg p-6 max-w-sm">
                        <h2 className="text-xl font-semibold mb-4">Parabéns!</h2>
                        <p>Você completou todos os vídeos deste curso!</p>
                        <button
                            onClick={handleClosePopup}
                            className="mt-4 text-white bg-[#E8C36A] hover:bg-[#F4A163] font-medium px-4 py-2 rounded">
                            Fechar
                        </button>
                    </div>
                </div>
            )}
        </div>
    );
};

export default CoursePage;
