import React, { useEffect, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import api from '../api';
import Sidebar from "../components/Sidebar";
import { toast } from 'react-toastify';

const AddCoursePage = () => {
    const [name, setName] = useState('');
    const [videos, setVideos] = useState([{ video_name: '', video_url: '', description: '', module: 1, order: 1 }]);
    const [message, setMessage] = useState('');
    const navigate = useNavigate();
    const [user, setUser] = useState(null);
    const [loading, setLoading] = useState(true);
    const [description, setDescription] = useState('');
    const [file, setFile] = useState(null);
    const [imagePreview, setImagePreview] = useState(null);


    useEffect(() => {
        const auth_token = localStorage.getItem("auth_token");
        const auth_token_type = localStorage.getItem("auth_token_type");
        const token = `${auth_token_type} ${auth_token}`;

        api
            .get("/users/", {
                headers: { Authorization: token },
            })
            .then((response) => {
                console.log(response.data); 
                setUser(response.data.result);
            })
            .catch((error) => {
                console.log(error);
            })
            .finally(() => {
                setLoading(false);
            });

    }, []);


    const handleAddVideo = () => {
        const lastVideo = videos[videos.length - 1];
        const newModule = lastVideo ? lastVideo.module : 1;
    
        setVideos([...videos, { 
            video_name: '', 
            video_url: '', 
            description: '', 
            module: newModule,
            order: videos.length + 1 
        }]);
    };

    const handleFileInputChange = (event) => {
        const selectedFile = event.target.files[0];
        setFile(selectedFile);
        if (selectedFile) {
            const reader = new FileReader();
            reader.onloadend = () => {
                setImagePreview(reader.result);
            };
            reader.readAsDataURL(selectedFile);
        }
    };

    
    const handleVideoChange = (index, field, value) => {
        const updatedVideos = videos.map((video, i) => {
            if (i === index) {
                if (field === 'module' || field === 'order') {
                    return { ...video, [field]: parseInt(value, 10) || 0 };  
                }
                return { ...video, [field]: value };
            }
            return video;
        });
        setVideos(updatedVideos);
    };

    const readFileAsBase64 = (file) => new Promise((resolve, reject) => {
        const reader = new FileReader();
        reader.onloadend = () => resolve(reader.result);
        reader.onerror = reject;
        reader.readAsDataURL(file);
    });
    
    const handleAddCourse = async (e) => {
        e.preventDefault();
        const auth_token = localStorage.getItem("auth_token");
        const auth_token_type = localStorage.getItem("auth_token_type");
        const token = `${auth_token_type} ${auth_token}`;
        
        const requestBody = {
            course_name: name,
            description: description,
            image: file ? await readFileAsBase64(file) : null,
            videos: videos.map(video => ({
                video_name: video.video_name,
                video_url: video.video_url,
                description: video.description,
                module: video.module,
                order: video.order,
            })),
        };

        console.log(requestBody);
        
        try {
            const response = await api.post('/courses/create', requestBody, {
                headers: { Authorization: token }
            });
    
            setMessage(response.data.message);
            toast.success("Course added successfully!");
            navigate("/");
        } catch (error) {
            const errorMessage = Array.isArray(error.response.data.detail)
                ? error.response.data.detail.map(err => `${err.loc[1]}: ${err.msg}`).join(', ')
                : error.response.data.detail || 'An error occurred';
            setMessage(errorMessage);
            toast.error(errorMessage);
        }
    };

    return (
        <div className="flex h-screen bg-white font-euphemia"  style={{ fontFamily: 'Euphemia UCAS Bold' }}>
            {!loading && <Sidebar user={user} />}
            
            <div className="flex-1 p-6 overflow-y-auto">
                <div className="bg-white p-6 rounded-lg shadow-md">
                    <h1 className="text-3xl font-bold text-center mb-6 text-[#E8C36A]">Adicionar Curso</h1>
                    <form onSubmit={handleAddCourse} className="space-y-6">
                        <div>
                            <label className="block font-medium mb-1 text-black">Nome do Curso</label>
                            <input
                                type="text"
                                className="w-full p-3 border rounded-lg shadow-sm focus:outline-none focus:ring focus:ring-[#F4A163]"
                                value={name}
                                onChange={(e) => setName(e.target.value)}
                                required
                            />
                        </div>
    
                        <div>
                            <label className="block font-medium mb-1 text-black">Descrição do Curso</label>
                            <textarea
                                className="w-full p-3 border rounded-lg shadow-sm focus:outline-none focus:ring focus:ring-[#F4A163]"
                                value={description}
                                onChange={(e) => setDescription(e.target.value)}
                                required
                            />
                        </div>
                        
                        <div>
                            <label className="block font-medium mb-1 text-black">Imagem do Curso</label>
                            <input
                                type="file"
                                className="w-full p-3 border rounded-lg shadow-sm focus:outline-none focus:ring focus:ring-[#F4A163]"
                                onChange={handleFileInputChange}
                                required
                            />
                            {imagePreview && <img src={imagePreview} alt="Image Preview" className="w-32 h-auto rounded-md" />}
                        </div>
    
                        <div>
                            <div className="max-h-60 overflow-y-scroll">
                                {videos.map((video, index) => (
                                    <div key={index} className="border-b mb-4 pb-4">
                                        <h2 className="text-lg font-semibold text-black">Vídeo {index + 1}</h2>
                                        <input
                                            type="text"
                                            className="w-full p-2 border rounded-lg mb-2"
                                            value={video.video_name}
                                            onChange={(e) => handleVideoChange(index, 'video_name', e.target.value)}
                                            placeholder="Nome do Vídeo"
                                            required
                                        />
                                        <input
                                            type="text"
                                            className="w-full p-2 border rounded-lg mb-2"
                                            value={video.video_url}
                                            onChange={(e) => handleVideoChange(index, 'video_url', e.target.value)}
                                            placeholder="URL do Vídeo"
                                            required
                                        />
                                        <input
                                            type="text"
                                            className="w-full p-2 border rounded-lg mb-2"
                                            value={video.description}
                                            onChange={(e) => handleVideoChange(index, 'description', e.target.value)}
                                            placeholder="Descrição"
                                        />
                                        <div className="flex space-x-4">
                                            <div className="w-1/2">
                                                <label className="block font-medium mb-1 text-black">Módulo</label>
                                                <input
                                                    type="number"
                                                    className="w-full p-2 border rounded-lg"
                                                    value={video.module}
                                                    onChange={(e) => handleVideoChange(index, 'module', parseInt(e.target.value, 10))}
                                                    placeholder="Número do Módulo"
                                                    min="1"
                                                    required
                                                />
                                            </div>
                                            <div className="w-1/2">
                                                <label className="block font-medium mb-1 text-black">Ordem</label>
                                                <input
                                                    type="number"
                                                    className="w-full p-2 border rounded-lg"
                                                    value={video.order}
                                                    onChange={(e) => handleVideoChange(index, 'order', parseInt(e.target.value, 10))}
                                                    placeholder="Ordem do Vídeo"
                                                    min="1"
                                                    required
                                                />
                                            </div>
                                        </div>
                                    </div>
                                ))}
                            </div>
                            <button
                                type="button"
                                onClick={handleAddVideo}
                                className="py-2 px-4 bg-[#E8C36A] text-black rounded-md hover:bg-[#F4A163] transition duration-200"
                            >
                                Adicionar Outro Vídeo
                            </button>
                        </div>
                        <button
                            type="submit"
                            className="w-full py-3 bg-[#E8C36A] text-black rounded-lg shadow-md hover:bg-[#F4A163] transition duration-200"
                        >
                            Adicionar Curso
                        </button>
                    </form>
                    {message && <p className="text-center mt-4 text-green-600">{message}</p>}
                </div>
            </div>
        </div>
    );
    
};

export default AddCoursePage;
